@@ -7,7 +7,7 @@ x0 <- rep.int(pi, 4)
 reschk <- function(res) {
     stopifnot(is.list(res),
               inherits(res, "minqa"),
-              names(res) == c("par", "fval", "feval"),
+              names(res) == c("par", "fval", "feval", "ierr", "msg"),
               is.numeric(res$par),
               all.equal(res$par, 1:4, tol = 2e-4),
               is.numeric(res$fval),
@@ -15,7 +15,7 @@ reschk <- function(res) {
               is.integer(res$feval),
               res$feval > 0)
 }
-
+# NOTE: we do not check ierr or msg here. JN 20100810
 reschk(ans.nd <- newuoa(x0, minfn, control = list(iprint = 2)))
 ans.nd
 reschk(ans.ud <- uobyqa(x0, minfn, control = list(iprint = 2)))
